<?php declare(strict_types = 1);

namespace Drupal\opencase;
use Drupal;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\opencase\Utils;

final class TimeBasedFieldUpdater {

    private EntityTypeManagerInterface $entityTypeManager;
    private string $date_field;
    private string $entity_type;
    private array $conditions;
    private string $date_format;

    final public function __construct(
        EntityTypeManagerInterface $entityTypeManager, 
        Utils $utils,
        string $entity_type, string $bundle, string $date_field, string $date_format = 'Y-m-d'
    )
    {   
        $this->entityTypeManager = $entityTypeManager;
        $this->utils = $utils;
        $this->date_field = $date_field;
        $this->date_format = $date_format;
        $this->entity_type = $entity_type;
        $this->bundle = $bundle;
    }

    final public function update(array $conditions, string $time_elapsed, array $new_values): void {
        $query = $this->entityTypeManager->getStorage($this->entity_type)->getQuery();
        $conditions[] = [$this->date_field, date($this->date_format, strtotime('-'.$time_elapsed)), "<"];
        $conditions[] = ['type', $this->bundle, '='];
        $this->utils->addConditionsToQuery($query, $conditions);
        foreach($query->execute() as $id) {
            $entity = $this->entityTypeManager->getStorage($this->entity_type)->load($id);
            foreach($new_values as $new_field=>$new_value) {
                $entity->$new_field = $new_value; 
            }
            $entity->save();
        }      
    }
}